import java.io.*;
import java.net.*;


class DBClient extends Thread{
    URL[] urlList;
    int numUrls;
    int period;
    int conTimeout;
    int queryTimeout;
    int retry;
    int retryDelay;

    Object data ;
    Object oldData;

    Socket sock;

    
    DBClient(String[] urlList, int numUrls, int period, int conTimeout, 
	    int queryTimeout, int retry, int retryDelay) {
	int port= 0;
	URL u = null;

	this.numUrls = numUrls;
	this.period = period; 
	this.conTimeout = conTimeout;
	this.queryTimeout = queryTimeout;
	this.retry = retry;
	this.retryDelay = retryDelay;
	this.urlList = new URL[256];

	for(int i=0; i < numUrls; i++) {
	    try{
		this.urlList[i] = new URL(urlList[i]);
	    } catch(MalformedURLException e) { 	    
		System.err.println(e);
	    }
	}

	start();
    }
    /**
     * Main function gets file of URLs and parameters and passes all to
     * instance of DBClient    
     */
    public static void main(String[] args) {
	BufferedReader br = null;
	String line = null;
	String[] urlList = new String[256];
	DBClient dbc = null;
	int numUrls = 0;

	// Get url list file
	if( args.length != 6) {
	    System.err.println("Usage: java DBClient url_list " +  
		    "period connection_timeout read_timeout retry retry_delay");
	    System.exit(1);
	}
	
	// Get url list file
	try { 
	    br = new BufferedReader( new FileReader(args[0]));
	} catch( FileNotFoundException e) {
	    System.err.println(e);
	    System.exit(1);
	}

	// Read url list file 
	do {
	    try {
		line = br.readLine();
		if ( line != null) {
		    urlList[numUrls++] = line;
		}
	    } catch( IOException e) {
		System.err.println(e);
	    }
	} while( line != null );

	// Run DBClient
	dbc = new DBClient(urlList, numUrls, Integer.parseInt(args[1]),
		Integer.parseInt(args[2]), Integer.parseInt(args[3]),
		Integer.parseInt(args[4]), Integer.parseInt(args[5]));
    }


    /**
     * Main procedure of DBClient
     */
    public void run()
    {
	BufferedReader in = null;
	PrintWriter out = null; 
	long startTime = 0; 
	int index = 0;

	/* Test 
	for(int i=0; i < numUrls; i++)
	    System.out.println( urlList[i]); 
	*/
	
	// Send query and receive data periodically 
	while(true) {
	    startTime = System.currentTimeMillis(); 

	    try {
		System.err.println("----------------------------------------------");
		index = connect(); 	// Fault tolerrant connection
		sock.setSoTimeout(queryTimeout);// Set timeout for receiving data

		in = new BufferedReader (new InputStreamReader( 
			sock.getInputStream()));
		out = new PrintWriter(new OutputStreamWriter(
			sock.getOutputStream()));

		// Send query to URL
		out.println("GET " + urlList[index].getFile());
		out.flush();
		System.err.println("Query sent :" + "GET " + urlList[index]);

		// Receive data from the URL
		data = getData(in);
		System.err.println("Data received :" + data);
		oldData = data;
		process(data);
	
		// Close the connection
		in.close();
		out.close();
		sock.close();
		sleep(period - (System.currentTimeMillis() - startTime));
	    } catch( InterruptedException e) {
		System.err.println(e);
	    } catch( IllegalArgumentException e1) {
		System.err.println(e1);
	    } catch( UnknownHostException e) {
		System.err.println(e);
	    } catch( IOException e) {
		System.err.println(e);
	    }
	}
    }

    int connect() {
	String url = null;
	String host = null;
	int port = 0;
	int index= 0;
	ConnectTry ct = null;

	// Select random url and read content from the url.
	while(true) {
	    index = (int)(Math.random() * numUrls);// 0 <= randome  < 1
	    host = urlList[index].getHost();
	    port = urlList[index].getPort(); 
	    port = (port == -1) ? 80 :port;	

	    ct = new ConnectTry(host,port);		
	    try {
		ct.join(conTimeout);
		if( ct.result() == null) {
		    System.err.println("Connection timeout(" + conTimeout 
					+ ") to " + urlList[index]);
		    ct.interrupt();
		    continue;
		}
		else {
		    sock = ct.result();
		}
		break;
	    } catch( InterruptedException e) {
	    }
	}

	return index;
    }

    Object getData(BufferedReader in) {
	int r = 0;

	while( r < retry) { 
	    try{
		data = in.readLine();
		oldData = data;
		return data;
	    } catch(InterruptedIOException e) {
		System.err.println("Query Timeout(" + queryTimeout+
				   ") happen!");
		try{ sleep( retryDelay);}
		catch(Exception e2) {}
		r ++;
	    } catch(IOException e3) {}

	}
	System.err.println("Old data is used due to max query failure" );
	return oldData;
    }


    Object process(Object o) {
	return null;
    }
}

class ConnectTry extends Thread {
    Socket s;
    String host;
    int port;

    ConnectTry(String host,int port) {
	this.host = host;
	this.port = port;
	start();
    }

    public Socket result() {
	return s;
    }
    public void run() {
	try{
	    s = new Socket(host,port);
	    if( this.currentThread().isInterrupted() ) {
		s.close();
	    }
	} 
	catch(InterruptedIOException e) { }
	catch( SocketException e) {System.err.println(e);}
	catch(UnknownHostException e) {System.err.println(e);}
	catch(IOException e) { System.err.println(e);}

    }
}

	
