/** DBClient.java 
 *
 *  Copyright (c) 2000
 *  All rights reserved.
 *  @author     Moonjoo Kim, Sampath Kannan, Insup Lee
 */

import java.io.*;
import java.net.*;
import java.util.*;

/** One connection at a time */
public class DBClient extends Thread{
    URL[] urlList;
    int numUrls;
    int period;

    int conRetryDelay;
    int conRetry;
    int conTimeout;

    int queryTimeout;
    int queryRetry;
    int queryRetryDelay;


    BufferedReader in ;
    PrintWriter out ; 

    Object data ;
    Object oldData;

    Socket sock;
    static int dumb = 9;

    
    DBClient(String[] urlList, int numUrls, int period, 
	    int conRetry, int conTimeout, int conRetryDelay, 
	    int queryRetry, int queryTimeout, int queryRetryDelay) {
	int port= 0;
	URL u = null;

	this.numUrls = numUrls;
	this.period = period; 

	this.conRetry= conRetry;
	this.conTimeout = conTimeout;
	this.conRetryDelay = conRetryDelay;

	this.queryTimeout = queryTimeout;
	this.queryRetry = queryRetry;
	this.queryRetryDelay = queryRetryDelay;
	this.urlList = new URL[256];

	for(int i=0; i < numUrls; i++) {
	    try{
		this.urlList[i] = new URL(urlList[i]);
	    } catch(MalformedURLException e) { 	    
		System.err.println(e);
	    }
	}

	// Initialize oldData and newData
	oldData = new Object();
	data = new Object();
	start();
    }
    /**
     * Main function gets file of URLs and parameters and passes all to
     * instance of DBClient    
     */
    public static void main(String[] args) {
	BufferedReader br = null;
	String line = null;
	String[] urlList = new String[256];
	DBClient dbc = null;

	int numUrls = 0;
	int period=0;
	int numCon=0;
	int conTimeOut=0;
	int conDelay=0;
	int numRead=0;
	int readTimeOut=0;
	int readDelay=0;

	//create gui
	//new DBClientGUI().init();

	// Get url list file
	if( args.length != 1) {
	    System.err.println("Usage: java DBClient config_file");
	     /*     url_list " +  
		    "period numCon conTimeout conDelay " + 
		    "numRead readTimeout readDelay"); 
	    */
	} else {
	
	    // Get url list file
	    try { 
		br = new BufferedReader( new FileReader(args[0]));
	    } catch( FileNotFoundException e) {
		System.err.println(e);
		System.exit(1);
	    }

	    // Read configuration line 
	    try {
		line = br.readLine();
		if ( line != null) {
		    StringTokenizer st = new StringTokenizer(line);
		    period= Integer.parseInt(st.nextToken()); 
		    numCon=Integer.parseInt(st.nextToken()); 
		    conTimeOut=Integer.parseInt(st.nextToken()); 
		    conDelay=Integer.parseInt(st.nextToken()); 
		    numRead=Integer.parseInt(st.nextToken()); 
		    readTimeOut=Integer.parseInt(st.nextToken()); 
		    readDelay=Integer.parseInt(st.nextToken()); 
		    System.out.println("Period 	                     = "+period +" ms");
		    System.out.println("# of Connection Retry        = "+numCon+" times");
		    System.out.println("Connection Timeout           = "+conTimeOut+" ms");
		    System.out.println("Delay Among Connection Retry = "+conDelay+" ms");
		    System.out.println("# of Read Retry              = "+numRead+" times");
		    System.out.println("Read Timeout                 = "+readTimeOut+" ms");
		    System.out.println("Delay Among Read Retry       = "+readDelay+" ms");
		}
		else {
		    System.err.println("Configuration Line is incorrect");
		}
	    } catch( IOException e) {
		System.err.println(e);
	    }

	    do {
		try {
		    line = br.readLine();
		    if ( line != null) {
			urlList[numUrls++] = line;
		    }
		} catch( IOException e) {
		    System.err.println(e);
		}
	    } while( line != null );

	    // Run DBClient
	    dbc = new DBClient(urlList, numUrls, period, numCon,
		    conTimeOut,conDelay, numRead, readTimeOut, readDelay); 
	}
    }


    /**
     * Main procedure of DBClient
     */
    public void run()
    {
	long startTime = System.currentTimeMillis(); 
	int index = 0;
	String s= null;

	/* Test 
	for(int i=0; i < numUrls; i++)
	    System.out.println( urlList[i]); 
	*/
	
	// Send query and receive data periodically 
	try {
	    System.err.println("----------------------------------------------");
	    index = connect(); 	// Fault tolerrant connection
	    if(index == -1 ) {// All connection try failed
		    System.err.println("Old data is used due to " + conRetry +" Connection failure!") ;
		processOldData();
	    } else {

		sock.setSoTimeout(queryTimeout);// Set timeout for receiving data

		in = new BufferedReader (new InputStreamReader( 
			sock.getInputStream()));
		out = new PrintWriter(new OutputStreamWriter(
			sock.getOutputStream()));

		// Send query to URL
		s = "GET " + urlList[index].getFile() + " HTTP/1.0\r\nHost: " + 
		    urlList[index].getHost() + "\r\n\r\n";
		out.print(s);
		out.flush();
		//System.err.println(s);
		System.err.println("Query sent :" + "GET " + urlList[index]);

		// Receive data from the URL
		data = getData();
		if( data == oldData) {
		    System.err.println("Old data is used due to " + queryRetry +
			" QUERY failures! ");
		    //System.err.println("Index : " + index);
		    processOldData();
		} else {
		    System.err.println("New Data received. " );
		    process();
		}
		in.close();
		out.close();
		sock.close();
	    }
	    oldData = data;

	    // Close the connection
	    //System.out.println("Time before sleep: " + System.currentTimeMillis() );
	    //System.out.println("Period sleeped : " +  (period -(System.currentTimeMillis() - startTime)));
	    sleep(period - (System.currentTimeMillis() - startTime));
	} catch( InterruptedException e) {
	    System.err.println(e);
	} catch( IllegalArgumentException e1) {
	} catch( UnknownHostException e) {
	    System.err.println(e);
	} catch( IOException e) {
	    System.err.println(e);
	}
	//System.out.println("Time after sleep: " + System.currentTimeMillis() );
	run();
    }
    void failConnection(ConnectTry ct) {
	ct.interrupt();
    }
    int connect() {
	String url = null;
	String host = null;
	int port = 0;
	int index= 0;
	ConnectTry ct = null;
	int numTryConn = 0;

	// Select random url and read content from the url.
	while(numTryConn < conRetry) { 
	    index = (int)(Math.random() * numUrls);// 0 <= randome  < 1
	    host = urlList[index].getHost();
	    port = urlList[index].getPort(); 
	    port = (port == -1) ? 80 :port;	

	    ct = new ConnectTry(host,port);		
	    try {
		ct.join(conTimeout);
		if( ct.result() == null) {
		    System.err.println("Connection timeout(" + conTimeout 
					+ ") to " + urlList[index]);
		    //System.err.println("DBClient uses oldData due to connection failure");
		    //ct.interrupt();
		    failConnection(ct);
		    numTryConn++;
		    try{
			Thread.sleep(conRetryDelay);
		    } catch(InterruptedException e) { System.err.println(e);}
		    continue;
		}
		else {
		    sock = ct.result();
		    return index;
		}
	    } catch( InterruptedException e) {
	    }
	}

	return -1;
    }

    Object getData() {
	//oldData = data;
	Object result;
	try{
	    result = in.readLine();
	    //return data;
	} catch(InterruptedIOException e) {
	    System.err.println("Query Timeout(" + queryTimeout+
			       ") happen!");
	    result = retryGetData(1);
	} catch(IOException e3) {
	    System.err.println(e3);
	    result = null;
	}
	return result;
    }

    Object retryGetData(int r) {

	if( r < queryRetry) { 
	    try{ sleep( queryRetryDelay);}
	    catch(Exception e2) {}

	    try{
		data = in.readLine();
		return data;
	    } catch(InterruptedIOException e) {
		System.err.println("Query Timeout(" + queryTimeout+
			       ") happen!");
		return retryGetData(r+1);
	    } catch(IOException e3) {System.err.println(e3);}
	}
	return oldData;
    }

    Object process() {
	return null;
    }
    Object processOldData() {
	return null;
    }

}

class ConnectTry extends Thread {
    Socket s;
    String host;
    int port;

    ConnectTry(String host,int port) {
	this.host = host;
	this.port = port;
	start();
    }

    public Socket result() {
	return s;
    }
    public void run() {
	try{
	    s = new Socket(host,port);
	    //s = new Socket("saul.cis.upenn.edu",8001);
	    if( this.currentThread().isInterrupted() ) {
		s.close();
	    }
	} 
	catch(InterruptedIOException e) { }
	catch( SocketException e) {System.err.println(e);}
	catch(UnknownHostException e) {System.err.println(e);}
	catch(IOException e) { System.err.println(e);}

    }
}
