
import java.lang.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.applet.*;
import javax.swing.*;

public class Console extends JApplet implements ActionListener {
  // UI components
  Air     air;
  AlarmIndicator patternAlarm, radiationAlarm;
  JPanel monitorDisplay;
  Box toolBar;
  JButton startButton, stopButton, restartButton, blastButton;
//  JButton radiationButton;
  JButton snapButton;
  Label prevL, currL;

  URL codeBase;
  String paramWidth, paramHeight, paramDistance, paramGravity;
  String paramFriction, paramFmax, paramSleep, paramNumMAVs;
  

  PrintStream snapFile;

  public Console() { }

  public static void main( String[] args ) {
    Console thisEm = new Console();
    MyContext stub = new MyContext();
    thisEm.setStub( stub );

    Thread.currentThread().setName( "Main" );

    int width = 350, height = 100; 	// default value

    for( int i=0; i<args.length; i++ ) {
      if( args[i].equalsIgnoreCase("-airwidth") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-airheight") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-distance") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-gravity") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-friction") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-fmax") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-sleep") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-numMAVs") ) {
	stub.setParameter( args[i].substring(1), args[++i] ); 
      } 
      else if( args[i].equalsIgnoreCase("-height") ) {
	i++; height = Integer.parseInt( args[i] );
      } 
      else if( args[i].equalsIgnoreCase("-width") ) {
	i++; width = Integer.parseInt( args[i] );
      } 
    }
    thisEm.init();

    JFrame top = new JFrame( "MAV Console" );
    top.getContentPane().add( thisEm );
    top.setSize( width, height );
    top.pack();
    top.setVisible(true);
    thisEm.start();
    thisEm.air.paint( thisEm.air.getGraphics() );
  }

  // standard applet methods
  public void init() {

    // extract code base
    codeBase = getCodeBase();
    // extract parameters
    paramWidth = getParameter( "airwidth" ).equals("") ?
		"700":getParameter( "airwidth" );
    paramHeight = getParameter( "airheight" ).equals("") ?
		"700":getParameter("airheight");
    paramDistance =  getParameter("distance").equals("") ?
		"50" : getParameter( "distance" ); 
    paramGravity = getParameter("gravity").equals("") ?
		"1200" : getParameter( "gravity" );
    paramFriction = getParameter("friction").equals("") ?
		"40" : getParameter( "friction" );
    paramFmax = getParameter("fmax").equals("") ? 
		"1" : getParameter( "fmax" ); 
    paramSleep = getParameter("sleep").equals("") ?
		"10" : getParameter( "sleep" ); 
    paramNumMAVs = getParameter("numMAVs").equals("") ? 
		"50" : getParameter( "numMAVs" );
    // do initialization
    construct();
  }

  // does real initialization - construct GUI, create MAVs and support objects
  public void construct() {

    // lay out buttons
    toolBar = Box.createHorizontalBox();

    // start, restart and stop are self-explanatory
    startButton = new JButton("Start");
    startButton.addActionListener(this);
    toolBar.add( startButton );
    restartButton = new JButton("Restart");
    restartButton.addActionListener(this);
    toolBar.add( restartButton );
    stopButton = new JButton("Stop");
    stopButton.addActionListener(this);
    toolBar.add( stopButton );

    // blast button - cause a blast to happen
    // create an image
    Image bombImage;
    try { 
      URL bombImageURL = new URL( codeBase, "bomb.gif" ); 
      bombImage = getImage( bombImageURL );
      blastButton = new JButton( new ImageIcon( bombImage ) );
    } catch( MalformedURLException e ) { 
      blastButton = new JButton( "Blast" );
    }
    blastButton.addActionListener(this);
    toolBar.add( blastButton );

//      // radiation attack button
//      try { 
//        URL radiationImageURL = new URL( codeBase, "radiation.gif" ); 
//        bombImage = getImage( radiationImageURL );
//        radiationButton = new JButton( new ImageIcon( bombImage ) );
//      } catch( MalformedURLException e ) { 
//        radiationButton = new JButton( "Radiation" );
//      }
//      radiationButton.addActionListener(this);
//      toolBar.add( radiationButton );

//      // snapshot button
//      snapButton = new JButton( "Capture" );
//      snapButton.addActionListener(this);
//      toolBar.add( snapButton );
    

    // insert the tool bar
    getContentPane().add( toolBar, BorderLayout.NORTH );

//      // alarms indicators in a box
//      Box alarmBox = Box.createHorizontalBox();
//      // create the pattern alarm indicator
//      patternAlarm = new AlarmIndicator( "Pattern", Color.red );
//      alarmBox.add( patternAlarm.getComponent() );
//      // create the radiation alarm indicator
//      radiationAlarm = new AlarmIndicator( "Radiation", Color.yellow );
//      alarmBox.add( radiationAlarm.getComponent() );
//      // add the box area to the applet
//      getContentPane().add( alarmBox, BorderLayout.CENTER );

    JFrame airFrame = new JFrame("MAV emulation");

    // initialize Air
    air = new Air( Integer.parseInt( paramWidth ),
		   Integer.parseInt( paramHeight ), 
		   Integer.parseInt( paramDistance ), 
		   Integer.parseInt( paramGravity ),
		   (new Double( paramFriction.trim() ).doubleValue())/100,
		   new Double( paramFmax.trim() ).doubleValue(), 
		   this );
    airFrame.getContentPane().add( air );
    airFrame.setSize(air.WIDTH,air.HEIGHT);
    airFrame.show();

    // now that Air is created, add it as listener to the blast button
    blastButton.addActionListener( air );

    // now, create a custom cursor for the blast
    // start with the image - mini version of the blast button image
    Cursor bombCursor;
    try { 
      URL bombImageURL = new URL( codeBase, "mini-bomb.gif" ); 
      bombImage = getImage( bombImageURL );
      bombCursor = Toolkit.getDefaultToolkit().
	createCustomCursor( bombImage, new Point(2,2), "BOMB_CURSOR" );
    } catch( MalformedURLException e ) { 
      System.err.println( "wrong URL" );
      bombCursor = new Cursor( Cursor.CROSSHAIR_CURSOR );
    } catch( IndexOutOfBoundsException e ) {
      System.err.println( "bad hot spot" );
      bombCursor = new Cursor( Cursor.CROSSHAIR_CURSOR );
    }
    air.setBombCursor( bombCursor );

  }

  public void start() {
    startButton.setEnabled( true );
    restartButton.setEnabled( false );
    stopButton.setEnabled( false );
    blastButton.setEnabled( true );
//      radiationButton.setEnabled( true );
  }

  public void stop() {
    air.stop();
//     snapFile.close();
  }

  // create the MAVs at random positions
  public void createMAVs( int num ) {

    // a linear pattern - for testing
//    for( int i = 0; i < 10; i++ ) {
//      // create new MAV
//      Point2D initPos = new Point2D.Double( i*air.DIST+20, 500 );
//      MAV newMAV = new MAV( this, initPos );
//      air.addMAV( newMAV );
//      newMAV.start();
//    }
    // uniform distribution of MAVs
//    for( int i = 0; i < num; i++ ) {
//      // create new MAV
//      Point2D initPos = 
//	new Point2D.Double( Math.random()*(air.WIDTH-20)+10, 
//			    Math.random()*(air.HEIGHT-20)+10 );
//      MAV newMAV = new MAV( this, initPos );
//      air.addMAV( newMAV );
//      newMAV.start();
//    }
    // gaussian distribution of MAVs
    Random rand = new Random();
    int mavPeriod = Integer.parseInt( paramSleep );
    for( int i = 0; i < num; i++ ) {
      // create new MAV
      Point2D initPos = 
	new Point2D.Double( rand.nextGaussian()*20.0 + air.WIDTH/2,
			    rand.nextGaussian()*20.0 + air.HEIGHT/2 );
      MAV newMAV = new MAV( air, initPos, mavPeriod );
      air.addMAV( newMAV );
      //newMAV.setPriority( newMAV.getPriority() - 1 );
      newMAV.start();
    }
  }

  // ActionListener interface
  public void actionPerformed( ActionEvent e ) {
    if (e.getSource() == stopButton) {
      System.out.println( "stop" );
      stop();
      startButton.setEnabled( true );
      stopButton.setEnabled( false );
      restartButton.setEnabled( false );
    } else if (e.getSource() == startButton) {
//       try{ 
// 	snapFile = new PrintStream( new FileOutputStream( "snapshot.tex" ) );
//       } catch( FileNotFoundException ex ) {
// 	System.out.println( "file not found" );
//       }
      snapFile = System.out;
      startButton.setEnabled( false );
      stopButton.setEnabled( true );
      restartButton.setEnabled( true );
      air.start();
      createMAVs( Integer.parseInt( paramNumMAVs ) );
    } else if (e.getSource() == restartButton) {
      System.out.println( "restart" );
      stop();
      air.start();
      createMAVs( Integer.parseInt( paramNumMAVs ) );
      startButton.setEnabled( false );
      stopButton.setEnabled( true );
      restartButton.setEnabled( true );
//      } else if (e.getSource() == radiationButton) {
//        // induce trigonometry failure into some MAVs
//        plotter.displayEvent( Color.blue );
//        double rate = Double.parseDouble( getParameter( "hitRate" ) );
//        for( Enumeration mavs = air.getMAVs(); mavs.hasMoreElements(); ) {
//  	MAV thisMAV = (MAV)mavs.nextElement();
//  	if (Math.random() < rate) 
//  	  thisMAV.radiationHit();
//        }
//        if (Math.random() < rate/2)
//  	base.radiationHit();
    } else if (e.getSource() == snapButton) {
      snapFile.println( "\\begin{figure}" );
      snapFile.println( "\\begin{picture}(200,150)" );
      for( Enumeration mavs = air.getMAVs(); mavs.hasMoreElements(); ) {
	MAV thisMAV = (MAV)mavs.nextElement();
	Point2D pos = thisMAV.getPosition();
	snapFile.print( "\\put("+(int)(pos.getX()/5)+"," );
	snapFile.println( (int)(pos.getY()/5)+")"+"{\\circle*{2}}" );
      }
      snapFile.println( "\\end{picture}" );
      snapFile.println( "\\end{figure}" );
    } 
  }

  // methods for updating display
  public void updateAlertCounts( int curr ) {
//      plotter.displayGraphPoint( curr, Plotter.LINE_COLOR );
  }
  public void displayPatternAlarm() {
//      patternAlarm.showAlarm();
//      plotter.displayEvent( Color.red );
  }
  public void displayRadiationAlarm() {
//      radiationAlarm.showAlarm();
//      plotter.displayEvent( Color.yellow );
  }


}; // class Console


class MyContext implements AppletStub, AppletContext {

  Hashtable parameters;

  public MyContext() { parameters = new Hashtable(); }

  public void setParameter( String name, String value ) {
    parameters.put( name, value );
  }

  // AppletStub methods
   public boolean isActive() { return true; }
   public URL getDocumentBase() { return null; }
   public URL getCodeBase() { 
     try {
       return new URL( "file:"+System.getProperty("user.dir")+"/" ); 
     } catch( MalformedURLException e ) {
       return null;
     }
   }
   public String getParameter(String name) { 
     if( parameters.containsKey(name) ) return (String)parameters.get(name);
     else return "";
   }

   public AppletContext getAppletContext() { return this; }
   public void appletResize(int width, int height) {}
   
   // AppletContext methods
   public AudioClip getAudioClip(URL url) { return null; }
   public Image getImage(URL url) { 
     String file = url.getFile();
     return Toolkit.getDefaultToolkit().getImage( file );
   }
   public Applet getApplet(String name) { return null; }
   public Enumeration getApplets() { return null; }
   public void showDocument(URL url) {}
   public void showDocument(URL url, String target) {}
   public void showStatus(String status) {}
}
