import java.lang.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

public class Plotter extends Canvas {

  JScrollPane toplevel;
  Vector       plottedLine;
  Vector       events;

  int          areaHeight;
  int          maxPos;
  double       scaleFactor;
  int          linePos;
  final int    dotSize = 5;

  public static final Color LINE_COLOR = Color.black;

  public Plotter( Console c, Color back, int width, int height, 
		 String plotName, boolean standalone ) {

    // create a drawing area with scrollbars (actually, only the horizontal one)
    areaHeight = height;
    setBackground( back );
    setSize( 600, height );
    toplevel = new JScrollPane( this );
//    toplevel.setPreferredSize( new Dimension(width, height) );

    // add borders
    Border raisedbevel = BorderFactory.createRaisedBevelBorder();
    Border loweredbevel = BorderFactory.createLoweredBevelBorder();
    Border compound = 
      BorderFactory.createCompoundBorder(raisedbevel, loweredbevel); 
    toplevel.setBorder( compound );

    // add a frame to display it in a separate window
    if( standalone ) {
      JFrame plotterFrame = new JFrame( plotName );
      plotterFrame.getContentPane().add( toplevel );
      plotterFrame.setSize( toplevel.getPreferredSize() );
      plotterFrame.show();
      plotterFrame.pack();
    }
    plottedLine = new Vector();
    events = new Vector();

    scaleFactor = -1;
    linePos = 0;
  }

  public synchronized void displayGraphPoint( int p, Color ptC ) {
    if( scaleFactor < 0 ) {
      // calculate scaling, assumption: we are plotting a decreasing function
      scaleFactor = (0.85*areaHeight-dotSize)/p;
    }
    drawDot( linePos, p, ptC );
    linePos += 2*dotSize;
    plottedLine.add( new PlottedEvent( p, ptC ) );
  }

  private void drawDot( int p1, int p2, Color ptC ) {
    Graphics g = getGraphics();
    g.setColor( ptC );
    g.fillOval(p1, areaHeight - dotSize - (int)(scaleFactor*p2), 
	       dotSize, dotSize);
  }

  public synchronized void displayEvent( Color evC ) {
    drawLine( linePos, evC );
    events.add( new PlottedEvent( linePos, evC ) );
  }

  private void drawLine( int pos, Color evC ) {
    Graphics g = getGraphics();
    g.setColor( evC );
    g.drawLine( pos, 0, pos, areaHeight );
  }

  public void reset() { 
    plottedLine.clear();
    events.clear();
    Dimension drSize = getSize();
    Graphics g = getGraphics();
    g.setColor( getBackground() );
    g.fillRect( 0, 0, drSize.width, drSize.height );

    scaleFactor = -1;
    linePos = 0;
  }

  public synchronized void paint( Graphics g ) { 
    Dimension drSize = getSize();
    g.setColor( getBackground() );
    g.fillRect( 0, 0, drSize.width, drSize.height );
    int savePos = linePos;

    // redraw line
    int pos = 0;
    for (Enumeration linEn = plottedLine.elements(); 
	 linEn.hasMoreElements(); ) {
      PlottedEvent nextDot = (PlottedEvent)linEn.nextElement();
      drawDot( pos, nextDot.getPos(), nextDot.getColor() );
      pos += 2*dotSize;
    }
    // redraw events
    for (Enumeration evtEn = events.elements(); evtEn.hasMoreElements(); ) {
      PlottedEvent nextEvt = (PlottedEvent)evtEn.nextElement();
      drawLine( nextEvt.getPos(), nextEvt.getColor() );
    }
  }

  public Component getComponent() { return toplevel; }

  public void emitPlot( OutputStream out, int increment ) { }
}; // class Plotter

class PlottedEvent {
  int    pos;
  Color  evtColor;

  public PlottedEvent( int p, Color c ) { 
    pos = p; evtColor = c;
  }
  public int getPos() { return pos; }
  public Color getColor() { return evtColor; }
}; // class PlottedEvent
